import actor, projectile, random

class Enemy:
    def __init__(self, eType, x, y):
        self.actor = actor.Actor()
        self.eType = eType
        if self.eType == "Robot":
            self.actor.loadSprite("dummy_stand1.png","stand1")
            self.actor.setPos(x,y)
            self.life = 50 # robot's default life is 100
            self.movingLeft = True # are we scrolling left?
        elif self.eType == "Defender":
            self.actor.loadSprite("dummy_stand1.png","stand1")
            self.actor.setPos(x,y)
            self.life = 20 # not very much life
            self.lookingLeft = True # which way we're looking
            self.sightXRange = 300
            self.aiCount = 0
            self.aiMax = 40 # some number used for switching AI states
            self.aiState = "Seek"
            self.shootSpeed = 3.0
            self.shootCnt = 60
            self.shootMax = 60
        elif self.eType == "Tower":
            self.actor.loadSprite("tower_stand1.png","stand1")
            self.actor.setPos(x,y)
            self.life = 100
            self.facingLeft = True
            self.shootSpeed = 3.0
            self.shootCnt = 0
            self.shootMax = 90
        elif self.eType == "SuperTower":
            self.actor.loadSprite("superTower_stand1.png","stand1")
            self.actor.setPos(x,y)
            self.life = 150
            self.facingLeft = True
            self.shootSpeed = 2.0
            self.shootCnt = 0
            self.shootMax = 120
        self.actor.setSprite("stand1")
        self.width = self.actor.rect.width
        self.height = self.actor.rect.height
        self.halfwidth = self.width/2
        self.halfheight = self.height/2
        self.upsideDown = False

    def update(self, curLevel, rx, ry, projList):
        if self.eType == "Robot":
            self.robotUpdate(curLevel,rx,ry)
        elif self.eType == "Defender":
            self.defenderUpdate(curLevel,rx,ry,projList)
        elif self.eType == "Tower":
            self.towerUpdate(rx,ry,projList)
        elif self.eType == "SuperTower":
            self.superTowerUpdate(rx,ry,projList)

    def defenderUpdate(self, curLevel, rx, ry, projList):
        # If our robot is too far away from the enemy, skip the AI update
        if ( abs(rx-self.actor.px) > 800 ):
            return
        #Do physics updates first!
        self.actor.update()
        
        if self.aiState == "Seek":
            self.actor.setVelocityX(0.0)
            if self.aiCount > self.aiMax: #heh, *2 is a cheat
                self.aiCount = 0
                # do something interesting
                if random.randint(0,3) == 1: # 25% chance of turning around
                    self.lookingLeft = not self.lookingLeft
            else:
                self.aiCount += 1
            # spot the player
            diffx = self.actor.px - rx
            diffy = self.actor.py - ry 
            if abs(self.actor.py-ry) < 200: # less than 200 tall pixels between us!
                if self.lookingLeft == True:
                    if diffx > 0 and diffx < self.sightXRange:
            #else we do not currently spot the player
                        self.aiState = "Attack"
                elif self.lookingLeft == False:
                    if diffx < 0 and diffx > -self.sightXRange:
                        self.aiState = "Attack"
        elif self.aiState == "Attack":
            if self.aiCount > self.aiMax:
                self.aiCount = 0
                # do something interesting
                if random.randint(0,3) == 3: # 25% chance to runaway!
                    self.aiState = "Runaway"
                    self.lookingLeft = not self.lookingLeft
            else:
                self.aiCount += 1
            if self.shootCnt > self.shootMax:
                if self.lookingLeft == True:
                    projList.append(projectile.Projectile("Tower",self.actor.px,self.actor.py+5,-self.shootSpeed,0.0))
                else:
                    projList.append(projectile.Projectile("Tower",self.actor.px,self.actor.py+5,self.shootSpeed,0.0))
                self.shootCnt = 0
            else:
                self.shootCnt += 1
        elif self.aiState == "Runaway":
            if self.aiCount > self.aiMax:
                self.aiCount = 0
                # do something interesting
                if random.randint(0,5) == 5:
                    self.aiState = "Seek"
                    self.actor.setVelocityX(0.0)
            else:
                self.aiCount += 1
            if self.lookingLeft == True:
                self.actor.setVelocityX(-2.0)
            elif self.lookingLeft == False:
                self.actor.setVelocityX(2.0)

        # Physics & AI?
        # Collision detection for feet
        colX = {}
        colY = {}

        actorX,actorY = self.actor.getPos()
        # we know that the robots width & height are 32!
        colX['bm'] = int(actorX + (self.halfwidth))
        colY['bm'] = int(actorY + (self.height))
        colX['tm'] = int(actorX + (self.halfwidth))
        colY['tm'] = int(actorY)
        colX['lm'] = int(actorX)
        colY['lm'] = int(actorY + (self.halfheight))
        colX['rm'] = int(actorX + (self.width))
        colY['rm'] = int(actorY + (self.halfheight))

        collisionOrder = ['lm','rm','tm','bm']

        for colName in collisionOrder:
            ty = (colY[colName]/32) * curLevel.getWidth()
            tx = (colX[colName]/32)
            if self.actor.inFlight() == True:
                tile = curLevel.data[3][(ty+tx)]
                if ( tile != 0 ): # 0 is universal empty?
                    # make this universal?
                    if ( colName == 'bm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.hitFloor()
                        else:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                    elif ( colName == 'tm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                        else:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            # -1 because we are technically moving the y down 1 more (y+32 when y is zero based!)
                            self.actor.hitFloor()
                    elif ( colName == 'lm' ):
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))-1
                    elif ( colName == 'rm' ):
                        self.actor.px = self.actor.px - (colX[colName] % 32)
            elif self.actor.inFlight() == False:
                if ( curLevel.data[3][(ty+tx)] == 0 ): # 0 is universal empty?
                    if self.actor.upsideDown == False:
                        if ( colName == 'bm' ):
                            if self.aiState != "Seek":
                                if self.lookingLeft == True:
                                    self.actor.px += 10
                                else:
                                    self.actor.px -= 10
                                self.lookingLeft = not self.lookingLeft
                                self.aiState = "Seek"
                    else:
                        if ( colName == 'tm' ):
                            if self.aiState != "Seek":
                                if self.lookingLeft == True:
                                    self.actor.px += 10
                                else:
                                    self.actor.px -= 10
                                self.lookingLeft = not self.lookingLeft
                                self.actor.px -= self.actor.vx
                                self.aiState = "Seek"
                elif ( curLevel.data[3][(ty+tx)] != 0 ): # tile is not empty
                    if ( colName == 'lm' ):
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))
                        if self.aiState != "Seek":
                            if self.lookingLeft == True:
                                self.actor.px += 10
                            else:
                                self.actor.px -= 10
                            self.lookingLeft = not self.lookingLeft
                            self.aiState = "Seek"
                    elif ( colName == 'rm' ):
                        self.actor.px = self.actor.px - (colX[colName] % 32)
                        if self.aiState != "Seek":
                            if self.lookingLeft == True:
                                self.actor.px -= 10
                            else:
                                self.actor.px += 10
                            self.lookingLeft = not self.lookingLeft
                            self.aiState = "Seek"

    def superTowerUpdate(self, rx, ry, projList):
        if ( abs(rx-self.actor.px) > 800 ):
            return
        elif ( abs(ry-self.actor.py) < 120 ):
            if self.shootCnt > self.shootMax:
                projList.append(projectile.Projectile("Tower",self.actor.px,self.actor.py+5,-self.shootSpeed,-0.5))
                projList.append(projectile.Projectile("Tower",self.actor.px,self.actor.py+5,-self.shootSpeed/2,self.shootSpeed/2))
                projList.append(projectile.Projectile("Tower",self.actor.px,self.actor.py+5,0.0,-self.shootSpeed))
                projList.append(projectile.Projectile("Tower",self.actor.px,self.actor.py+5,self.shootSpeed/2,self.shootSpeed/2))
                projList.append(projectile.Projectile("Tower",self.actor.px,self.actor.py+5,self.shootSpeed,-0.5))
                self.shootCnt = 0
            else:
                self.shootCnt += 1
        else:
            self.shootCnt = self.shootMax

    def towerUpdate(self, rx, ry, projList):
        if ( abs(rx-self.actor.px) > 800 ):
            return
        elif ( abs(ry-self.actor.py) < 90 ):
            if self.shootCnt > self.shootMax:
                if self.facingLeft == True:
                    projList.append(projectile.Projectile("Tower",self.actor.px,self.actor.py+5,-self.shootSpeed,0.0))
                else:
                    projList.append(projectile.Projectile("Tower",self.actor.px+32,self.actor.py+5,self.shootSpeed,0.0))
                self.shootCnt = 0
            else:
                self.shootCnt += 1
        else:
            self.shootCnt = self.shootMax

    def robotUpdate(self, curLevel, rx, ry):
        # If our robot is too far away from the enemy, skip the AI update
        if ( abs(rx-self.actor.px) > 800 ):
            return
        if self.actor.inFlight() == False:
            if self.movingLeft == True:
                self.actor.setVelocityX(-2.0)
            elif self.movingLeft == False:
                self.actor.setVelocityX(2.0)
        else:
            self.actor.setVelocityX(0.0)
        self.actor.update()
        # Physics & AI?
        # Collision detection for feet
        colX = {}
        colY = {}

        actorX,actorY = self.actor.getPos()
        # we know that the robots width & height are 32!
        colX['bm'] = int(actorX + (self.halfwidth))
        colY['bm'] = int(actorY + (self.height))
        colX['tm'] = int(actorX + (self.halfwidth))
        colY['tm'] = int(actorY)
        colX['lm'] = int(actorX)
        colY['lm'] = int(actorY + (self.halfheight))
        colX['rm'] = int(actorX + (self.width))
        colY['rm'] = int(actorY + (self.halfheight))

        collisionOrder = ['lm','rm','tm','bm']

        for colName in collisionOrder:
            ty = (colY[colName]/32) * curLevel.getWidth()
            tx = (colX[colName]/32)
            if self.actor.inFlight() == True:
                tile = curLevel.data[3][(ty+tx)]
                if ( tile != 0 ): # 0 is universal empty?
                    # make this universal?
                    if ( colName == 'bm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.hitFloor()
                        else:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                    elif ( colName == 'tm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                        else:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            # -1 because we are technically moving the y down 1 more (y+32 when y is zero based!)
                            self.actor.hitFloor()
                    elif ( colName == 'lm' ):
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))-1
                    elif ( colName == 'rm' ):
                        self.actor.px = self.actor.px - (colX[colName] % 32)
            elif self.actor.inFlight() == False:
                if ( curLevel.data[3][(ty+tx)] == 0 ): # 0 is universal empty?
                    if self.actor.upsideDown == False:
                        if ( colName == 'bm' ):
                            #self.actor.fall()
                            self.movingLeft = not self.movingLeft
                    else:
                        if ( colName == 'tm' ):
                            #self.actor.fall()
                            self.movingLeft = not self.movingLeft
                elif ( curLevel.data[3][(ty+tx)] != 0 ): # tile is not empty
                    if ( colName == 'lm' ):
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))
                        self.movingLeft = not self.movingLeft
                    elif ( colName == 'rm' ):
                        self.actor.px = self.actor.px - (colX[colName] % 32)
                        self.movingLeft = not self.movingLeft


    def draw(self, screen, camX, camY):
        if self.eType == "Robot":
            self.actor.draw(screen, camX, camY, self.movingLeft, self.actor.upsideDown)
        elif self.eType == "Defender":
            self.actor.draw(screen, camX, camY, self.lookingLeft, self.actor.upsideDown)
        elif self.eType == "Tower" or self.eType == "SuperTower":
            self.actor.draw(screen, camX, camY, self.facingLeft, self.actor.upsideDown)
    def takeDamage(self, damage, selfList):
        self.life -= damage
        if self.life <= 0:
            selfList.remove(self)
