import tile, data, enemy, gem

class Level:
    def __init__(self, name, enemyList, eventList):
        self.data = self.load(name)
        self.name = self.data[0]
        self.width = self.data[1]
        self.height = self.data[2]
        self.tileList = []
        self.initTiles()
        self.initEnemies(enemyList)
        self.initEvents(eventList)
        if self.name == "Jungle":
            self.backBG,null = data.load_image("jungleBG.png")
            self.frontBG,null = data.load_image("jungleFG.png",(0,255,0))
        elif self.name == "Space":
            self.backBG,null = data.load_image("spaceBG.png")
            self.frontBG,null = data.load_image("spaceFG.png",(0,255,0))
        elif self.name == "Lava":
            self.backBG,null = data.load_image("lavaBG.png")
            self.frontBG,null = data.load_image("lavaFG.png",(0,255,0))
        elif self.name == "War":
            self.backBG,null = data.load_image("warBG.png")
            self.frontBG,null = data.load_image("warFG.png",(0,255,0))
        elif self.name == "Final":
            self.backBG,null = data.load_image("jungleBG.png")
            self.frontBG,null = data.load_image("jungleFG.png",(0,255,0))

    def getWidth(self):
        return self.width

    def getHeight(self):
        return self.height
       
    def draw(self, screen, camX, camY):
        screen.blit(self.backBG,(0,0))
        screen.blit(self.frontBG,(-camX/8,0))
        camX32 = int(camX/32)
        camY32 = int(camY/32)
        for y in range(0,16): # 480/32 + 32
            for x in range(0,21): # 640/32 + 32
                index = (x+camX32)+((y+camY32)*self.width)
                if index < ((self.width * self.height)):
                    if self.tileList[index] != 0:
                        self.tileList[index].draw(screen,camX,camY)

    def initTiles(self):
        for y in range(0,self.height):
            for x in range(0,self.width):
                index = (x+y*self.width)
                if self.data[3][index] > 0:
                    self.tileList.append(tile.Tile(self.name,self.data[3][index],x*32,y*32))
                else:
                    self.tileList.append(0)

    def initEnemies(self, enemyList):
        for y in range(0,self.height):
            for x in range(0,self.width):
                index = (x+y*self.width)
                enemyNum = self.data[4][index]
                if enemyNum > 0:
                    if self.name == "Jungle":
                        if enemyNum == 1:
                            enemyList.append(enemy.Enemy("Robot",x*32,y*32))
                        elif enemyNum == 2:
                            enemyList.append(enemy.Enemy("SuperTower",x*32,y*32-32))
                        elif enemyNum == 3:
                            enemyList.append(enemy.Enemy("Defender",x*32,y*32))
                    if self.name == "Space":
                        if enemyNum == 1:
                            enemyList.append(enemy.Enemy("Robot",x*32,y*32))
                        elif enemyNum == 2:
                            newRobot = enemy.Enemy("Robot",x*32,y*32)
                            newRobot.actor.upsideDown = True
                            enemyList.append(newRobot)
                        elif enemyNum == 3:
                            enemyList.append(enemy.Enemy("Tower",x*32,y*32))
                        elif enemyNum == 4:
                            newTower = enemy.Enemy("Tower",x*32,y*32)
                            newTower.actor.upsideDown = True
                            enemyList.append(newTower)
                        elif enemyNum == 5:
                            newTower = enemy.Enemy("Tower",x*32,y*32)
                            newTower.facingLeft = False
                            enemyList.append(newTower)
                        elif enemyNum == 6:
                            newTower = enemy.Enemy("Tower",x*32,y*32)
                            newTower.facingLeft = False
                            newTower.actor.upsideDown = True
                            enemyList.append(newTower)
                    if self.name == "Lava":
                        if enemyNum == 1:
                            enemyList.append(enemy.Enemy("Robot",x*32,y*32))
                        elif enemyNum == 2:
                            enemyList.append(enemy.Enemy("Tower",x*32,y*32))
                        elif enemyNum == 3:
                            enemyList.append(enemy.Enemy("SuperTower",x*32,y*32-32))
                        elif enemyNum == 4:
                            enemyList.append(enemy.Enemy("Defender",x*32,y*32))
                    if self.name == "War":
                        if enemyNum == 1:
                            enemyList.append(enemy.Enemy("Robot",x*32,y*32))
                        elif enemyNum == 2:
                            enemyList.append(enemy.Enemy("Tower",x*32,y*32))
                        elif enemyNum == 3:
                            enemyList.append(enemy.Enemy("SuperTower",x*32,y*32-32))
                        elif enemyNum == 4:
                            enemyList.append(enemy.Enemy("Defender",x*32,y*32))


    def initEvents(self, eventList):
        for y in range(0,self.height):
            for x in range(0,self.width):
                index = (x+y*self.width)
                if self.data[5][index] == 1:
                    eventList["start"] = [x*32,y*32]
                elif self.data[5][index] == 2:
                    if self.name == "Jungle":
                        eventList["gem"] = gem.Gem("blue",x*32,y*32)
                    elif self.name == "Space":
                        eventList["gem"] = gem.Gem("red",x*32,y*32)
                    elif self.name == "Lava":
                        eventList["gem"] = gem.Gem("green",x*32,y*32)
                    elif self.name == "War":
                        eventList["gem"] = gem.Gem("yellow",x*32,y*32)

    def getTileAt(self, x, y):
        x32 = x/32
        y32 = y/32
        print "Tile List[%i][%i]"%(x32,y32)
        index = int(x/32 + (y/32)*self.width)
        tile = self.tileList[index]
        if tile != 0:
            return tile.getType()
        else:
            return "Air"
    
    def load(self, levelName):
        FILE = data.load(levelName)
        mapType = FILE.readline()
        mapType = mapType.strip("\r\n")
        mapWidth = int(FILE.readline())
        mapHeight = int(FILE.readline())
        baseLayer = []
        baseLine = FILE.readline()
        baseLine=baseLine.strip("\r\n")
        for num in baseLine:
            baseLayer.append(int(num))
        enemyLayer = []
        enemyLine = FILE.readline()
        enemyLine=enemyLine.strip("\r\n")
        for enemy in enemyLine:
            enemyLayer.append(int(enemy))
        eventLayer = []
        eventLine = FILE.readline()
        eventLine=eventLine.strip("\r\n")
        for event in eventLine:
            eventLayer.append(int(event))
        FILE.close()
        rLevel = [mapType,mapWidth,mapHeight,baseLayer,enemyLayer,eventLayer]
        return rLevel
