import pygame, data, gem
from pygame.locals import *

class Menu:
    def __init__(self,screen, clock, gems):
        self.screen = screen
        self.clock = clock
        self.background = pygame.Surface(self.screen.get_size())
        self.background = self.background.convert()
        self.background.fill((0,0,0))
        self.menuBG, null = data.load_image('menu.png')
        self.selectBG, null = data.load_image('stageSelect.png')
        self.instructions, null = data.load_image('instructions.png')
        self.ball, null = data.load_image('selectBall.png',(0,255,0))
        self.selectFrame,null = data.load_image('stageCursor.png',(0,255,0))
        self.finalLevel,null = data.load_image('finalLevel.png')
        self.selection = 0
        self.state = "Menu"
        self.enterPressed = False
        self.unlockFinish = False
        if len(gems) <= 3:
            self.maxLevel = 3 # default is the first 4 levels
        elif len(gems) == 4:
            # DEBUG ONLY FOR BETA 1
            self.maxLevel = 3
            #self.maxLevel = 4
            self.unlockFinish = True
        gemList = ["blue","red","green","yellow"]
        self.menuGems = []
        for g in gems:
            if g.getColor() == "blue":
                self.menuGems.append(gem.Gem("blue",0, 150))
            elif g.getColor() == "red":
                self.menuGems.append(gem.Gem("red",608, 150))
            elif g.getColor() == "green":
                self.menuGems.append(gem.Gem("green",0, 300))
            elif g.getColor() == "yellow":
                self.menuGems.append(gem.Gem("yellow",608,300))

    def update(self):
        rvalue = self.logic()
        self.render()
        self.clock.tick(60)
        return rvalue

    def logic(self):
        for event in pygame.event.get():
            if event.type == QUIT:
                return "Quit"
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    return "Quit"
                elif event.key == K_DOWN:
                    if self.state == "Menu":
                        self.selection+=1
                        if self.selection > 2:
                            self.selection = 0
                    elif self.state == "LevelSelect":
                        self.selection +=2
                        if self.selection > self.maxLevel:
                            self.selection = 0
                elif event.key == K_UP:
                    if self.state == "Menu":
                        self.selection-=1
                        if self.selection < 0:
                            self.selection = 2
                    elif self.state == "LevelSelect":
                        self.selection-=2
                        if self.selection < 0:
                            self.selection = self.maxLevel
                elif event.key == K_LEFT:
                    if self.state == "LevelSelect":
                        self.selection-=1
                        if self.selection < 0:
                            self.selection = self.maxLevel
                elif event.key == K_RIGHT:
                    if self.state == "LevelSelect":
                        self.selection+=1
                        if self.selection > self.maxLevel:
                            self.selection = 0
                elif event.key == K_RETURN:
                    self.enterPressed = True
            elif event.type == KEYUP:
                if event.key == K_RETURN:
                    self.enterPressed = False
        if self.state == "Menu":
            if self.enterPressed == True:
                if self.selection == 0:
                    self.state = "LevelSelect"
                    self.enterPressed = False
                elif self.selection == 1:
                    self.state = "Instructions"
                    self.enterPressed = False
                if self.selection == 2:
                    return "Quit"
        elif self.state == "LevelSelect":
            if self.enterPressed == True:
                if self.selection == 0:
                    return "Jungle"
                elif self.selection == 1:
                    return "Space"
                elif self.selection == 2:
                    return "Lava"
                elif self.selection == 3:
                    return "War"
                elif self.selection == 4:
                    return "Final"
        elif self.state == "Instructions":
            if self.enterPressed == True:
                self.state = "Menu"
                self.enterPressed = False
        return "Idle"

    def render(self):
        self.screen.blit(self.background, (0,0))
        if self.state == "Menu":
            self.screen.blit(self.menuBG, (0,0))
            self.screen.blit(self.ball, (215,218+self.selection*32))
        elif self.state == "LevelSelect":
            self.screen.blit(self.selectBG, (0,0))
            #if self.maxLevel == 4:
            if self.unlockFinish == True:
                self.screen.blit(self.finalLevel, (215,170))
            if self.selection == 0:
                self.screen.blit(self.selectFrame, (0, 0))
            elif self.selection == 1:
                self.screen.blit(self.selectFrame, (429, 0))
            elif self.selection == 2:
                self.screen.blit(self.selectFrame, (0, 337))
            elif self.selection == 3:
                self.screen.blit(self.selectFrame, (429, 337))
            elif self.selection == 4:
                self.screen.blit(self.selectFrame, (213, 168))
            for g in self.menuGems:
                g.draw(self.screen,0,0)
        elif self.state == "Instructions":
            self.screen.blit(self.instructions, (0,0))
            self.screen.blit(self.ball, (530, 443))
        pygame.display.flip()
